<?php
/*
 (c) 2015 Mobunti Limited, All Rights Reserved.

  File: postcodes.php
  Version: 6.0
  Date Last Modified: 26th Aug 2015

    This is an example "proxy" file. It could be rewritten easily in ASP or Java etc but it is important
    that it remain a server side script. If the public can see your access token they can use your postcode lookups
    for free

    Even though the code below is 150+ lines long, in essence it is 2 small calls to a simple url - the rest is just
    there to give you a mechanism to see what comes back from the server

    Remember to change the $token to your own

    if you want to use the same script to handle multiple postcode calls - so that you can track whether your
    contact us page receives more lookups than your order page, do not pass the token into this script.
    Instead pass through a parameter called, for example, "calltype" and set it to contactus or order etc
    and then have a small piece of code in this server side script which checks for calltype and then defines
    the correct token to use based on its setting


    PLEASE NOTE: THIS IS ONLY A  "SAMPLE" SCRIPT. YOU CAN READ AND UNDERSTAND THE REQUIREMENTS AND WRITE YOUR OWN SCRIPT
    OR USE THIS AS A BASE AND TWEAK IT TO YOUR REQUIREMENTS
*/


$token="TEST-TOKEN2";

class postcodeAPI
{
    const SERVICE="https://api.mobunti.net";
    public $error="";
    public $json = array();
    public $body="";
    public $headers= array();
    public $headerstring= "";
    public $httpCode = "";
    public $systemerrorcode = "";
    public $systemerror;

    public function http_parse_headers( $header )
    {
        $retVal = array();
        $fields = explode("\r\n", preg_replace('/\x0D\x0A[\x09\x20]+/', ' ', $header));
        foreach( $fields as $field ) {
            if( preg_match('/([^:]+): (.+)/m', $field, $match) ) {
                $match[1] = preg_replace('/(?<=^|[\x09\x20\x2D])./e', 'strtoupper("\0")', strtolower(trim($match[1])));
                if( isset($retVal[$match[1]]) ) {
                    $retVal[$match[1]] = array($retVal[$match[1]], $match[2]);
                } else {
                    $retVal[$match[1]] = trim($match[2]);
                }
            }
        }
        return $retVal;
    }

    public function PAF($command,$token,$parameter="")
    {
        try
        {
            if (!$token)
            {
                throw new Exception("No token provided");
            }
            if ($token=="TEST-TOKEN")
            {
                throw new Exception("The token used is the default one. You must update this script and provide your own");
            }
            if (!$command)
            {
                throw new Exception("No command provided. Please provide 'summary' or 'detail' or one of the admin commands as required");
            }

            if ($parameter>"") $parameter = "/".urlencode($parameter);
            $service_url = self::SERVICE."/postcodes/".$command.$parameter;
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL,$service_url);
            curl_setopt($curl, CURLOPT_POST, false);

            /*
             *  Note: This example script does not verify the ssl certificate
             *
             * Please follow appropriate instructions to verify the integrity of the ssl
             * certificate
             *
             *
             */
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT ,15);
            curl_setopt($curl, CURLOPT_TIMEOUT, 30); //timeout in seconds
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_VERBOSE, 1);
            curl_setopt($curl, CURLOPT_HEADER, 1);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('X-Postcode-Auth: '.$token));

            $response = curl_exec ($curl);

            $curl_errno = curl_errno($curl);
            $curl_error = curl_error($curl);

            $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);
            $header = substr($response, 0, $header_size);
            $body = substr($response, $header_size);
            $parse=$this->http_parse_headers($header);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if ($curl_errno > 0) {
                throw new Exception ("Interface Error ($curl_errno): $curl_error");
            }

            $this->headerstring = $header;
            $this->json = json_decode($body);
            $this->headers = $parse;
            $this->body = $body;
            $this->httpCode = $httpCode;

            if($httpCode == 404) {
                /* Handle 404 here. */
                throw new Exception("Postcode service not found. Please check settings");
            }
            elseif($httpCode == 403) {
                /* Handle 403 here. */
                if ($this->json)
                {
                    if ($this->json->error)
                    {
                        $this->systemerrorcode = $this->json->error;
                        $this->systemerror = $this->json->errordetail;
                        if ($this->systemerrorcode=="NOTFOUND-POSTCODE")
                            throw new Exception("The postcode you provided was not found. Please check and try again");
                        else
                        {
                            // any error raised here isnt really a user error so you might choose to send yourself
                            // an email
                            throw new Exception($this->systemerror);
                        }
                    }
                }

                throw new Exception("There seems to be a problem but we have no specific details. Please try again");
            }
            else {
                if ($httpCode != 200) {
                    /* Handle non 200 here. */
                    throw new Exception("We received an error $httpCode message");
                }
            }
        }
        catch(Exception $e)
        {
            $this->error = $e->getMessage();
            throw new Exception($this->error);
        }
    }
}

if (array_key_exists("postcode",$_REQUEST))
{
    $pcapi = new postcodeAPI();
    try
    {
        $pcapi->PAF("summary",$token,$_REQUEST["postcode"]);
        if ($pcapi->body)  echo ($pcapi->body);
        else throw new Exception("No response received. Please try again");
    }
    catch(Exception $e)
    {
        // the Exception is potentially a generic message but if it was caused by the postcode system itself
        // i.e. not a timeout or invalid command etc, then systemerror and systemerrorcode will give more information
        if ($pcapi->systemerrorcode>"")
        {
            echo "{\"error\": \"".$pcapi->systemerror."\",
             \"code\": \"".$pcapi->systemerrorcode."\"}";
        }
        else echo "{\"error\": \"".$e->getMessage()."\" }";
    }
}

if (array_key_exists("detail",$_REQUEST))
{
    $pcapi = new postcodeAPI();
    try
    {
        $pcapi->PAF("detail",$token,$_REQUEST["detail"]);
        if ($pcapi->body)  echo ($pcapi->body);
        else throw new Exception("No response received. Please try again");
    }
    catch(Exception $e)
    {
        // the Exception is potentially a generic message but if it was caused by the postcode system itself
        // i.e. not a timeout or invalid command etc, then systemerror and systemerrorcode will give more information
        if ($pcapi->systemerrorcode>"")
        {
            echo "{\"error\": \"".$pcapi->systemerror."\",
             \"code\": \"".$pcapi->systemerrorcode."\"}";
        }
        else echo "{\"error\": \"".$e->getMessage()."\" }";
    }
}
